package org.mmga.clubs.service;

import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.codec.digest.DigestUtils;
import org.mmga.clubs.dao.UserDao;
import org.mmga.clubs.entities.BaseResponse;
import org.mmga.clubs.entities.user.*;
import org.mmga.clubs.utils.JwtUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UserService {
    private final UserDao userDao;
    private final AuthService authService;
    private final JwtUtils jwtUtils;
    @Autowired
    public UserService(UserDao userDao, AuthService authService, JwtUtils jwtUtils){
        this.userDao = userDao;
        this.authService = authService;
        this.jwtUtils = jwtUtils;
    }

    public BaseResponse<Boolean> login(UserLoginVo user, HttpServletResponse response) {
        UserVo userVo = userDao.getUser(user.username(), DigestUtils.sha1Hex(user.password()));
        User u = packageUser(userVo);
        if (response != null && u != null){
            response.addHeader("Set-Authorization", jwtUtils.createToken(u));
        }
        return u == null ? BaseResponse.failed(404, "无效用户") : BaseResponse.success(true);
    }

    public BaseResponse<Boolean> createUser(UserRegVo user, HttpServletResponse response) {
        String username = user.username();
        if (userDao.countUser(username) > 0) {
            return BaseResponse.failed(409, "用户已存在");
        }
        UserRegResponseVo responseVo = new UserRegResponseVo(user);
        userDao.addUser(responseVo);
        int newUserId = responseVo.getId();
        User newUser = packageUser(userDao.getUserById(newUserId));
        if (newUser != null && response != null) {
            response.addHeader("Set-Authorization", jwtUtils.createToken(newUser));
        }
        return BaseResponse.success(true);
    }
    public User getUserById(int id){
        return packageUser(userDao.getUserById(id));
    }
    public User getGuestUser(){
        User user = new User();
        user.setId(-1);
        user.setName("游客");
        user.setAuth(authService.getAuthById(1));
        return user;
    }
    private User packageUser(UserVo vo) {
        if (vo == null){
            return null;
        }
        User result = new User();
        result.setId(vo.id());
        result.setName(vo.name());
        result.setAuth(authService.getAuthById(vo.auth()));
        return result;
    }

    public BaseResponse<User> getUserInfo(int userId) {
        if (userId == -1){
            return BaseResponse.success(getGuestUser());
        }
        return BaseResponse.success(getUserById(userId));
    }
}
